package de.intevation.mxd.reader;

import org.apache.log4j.Logger;

import com.esri.arcgis.carto.IFeatureRenderer;
import com.esri.arcgis.display.ISymbol;
import com.esri.arcgis.carto.UniqueValueRenderer;

import org.w3c.dom.Element;
import java.io.IOException;

import de.intevation.mxd.utils.MapToXMLUtils;

/**
 * Reads unique vaule renderer information.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class UniqueValueRendererReader
implements IRendererReader {

    /**
     * The logger.
     */
    private static final Logger logger =
        Logger.getLogger(FeatureLayerReader.class);

    /**
     * Private Member.
     */
    private UniqueValueRenderer renderer;
    private Element layer;
    private MapToXMLUtils util;


    public UniqueValueRendererReader(IFeatureRenderer renderer)
    throws Exception {
        logger.debug("constructor()");
        if (renderer instanceof UniqueValueRenderer) {
            this.renderer = (UniqueValueRenderer)renderer;
        }
        else {
            throw new Exception ("Not an instance of UniqueValueRenderer.");
        }
    }

    /**
     * Setter for the parent XML element.
     *
     * @param parent The XML parent node.
     */
    public void setParent(Element parent) {
        this.layer = parent;
    }

    /**
     * Setter for XML document helper.
     *
     * @param util The helper class for storing map information.
     */
    public void setUtil(MapToXMLUtils util) {
        this.util = util;
    }

    /**
     * Reads the renderer attributes.
     *
     * @return The parent XML node.
     */
    public Element read() {
        logger.debug("read()");
        int count = 0;
        try {
            count = renderer.getClassCount();
        }
        catch(IOException ioe) {
            logger.error(
                "Could not read class count."+
                " Stopped reading renderer.");
            return layer;
        }
        for(int i = 0; i < count; i++) {
            Element rendererElement = util.addRenderer(layer);

            try {
                String value = renderer.getValue(i);
                value = value.replaceAll(" ", "");
                if(value.indexOf(",") > 0) {
                    int j = 0;
                    while(value.indexOf(",") > 0) {
                        String v = value.substring(0, value.indexOf(","));
                        rendererElement.setAttribute("value_" + j, v);
                        value = value.substring(
                            value.indexOf(",") + 1,
                            value.length());
                        j++;
                    }
                    rendererElement.setAttribute("value_" + j, value);
                }
                else {
                    rendererElement.setAttribute("value", value);
                }

                value = renderer.getValue(i);
                rendererElement.setAttribute("name", renderer.esri_getClass(i));
                rendererElement.setAttribute("description",
                    renderer.getDescription(value));

                rendererElement.setAttribute("label", renderer.getLabel(value));
                rendererElement.setAttribute(
                    "field_count",
                    String.valueOf(renderer.getFieldCount()));
                for(int j = 0; j < renderer.getFieldCount(); j++) {
                    rendererElement.setAttribute(
                        "expression_field_" + j,
                        renderer.getField(j));
                }
                rendererElement.setAttribute("expression_operator", "=");

                ISymbol sym = renderer.getSymbol(value);
                MarkerSymbolReader markerReader = new MarkerSymbolReader();
                LineSymbolReader lineReader = new LineSymbolReader();
                FillSymbolReader fillReader = new FillSymbolReader();
                if(markerReader.canRead(sym)) {
                    markerReader.setSymbol(sym);
                    markerReader.setUtil(util);
                    markerReader.setParent(rendererElement);
                    markerReader.read();
                }
                else if(lineReader.canRead(sym)) {
                    lineReader.setSymbol(sym);
                    lineReader.setUtil(util);
                    lineReader.setParent(rendererElement);
                    lineReader.read();
                }
                else if(fillReader.canRead(sym)) {
                    fillReader.setSymbol(sym);
                    fillReader.setUtil(util);
                    fillReader.setParent(rendererElement);
                    fillReader.read();
                }
                else {
                    logger.debug("Not a known symbol type: " +
                                 sym.getClass().toString());
                }
            }
            catch(Exception e) {
                logger.error(
                    "Could not read unique value data." +
                    e.toString());
                return layer;
            }
        }
        return layer;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
